<?php
class Platron { public const PAY_TTL = 3600; protected $merchant = ''; protected $key = ''; protected $payUrl = 'https://www.platron.ru/init_payment.php'; protected $captureUrl = 'https://www.platron.ru/do_capture.php'; protected $reverseUrl = 'https://www.platron.ru/revoke.php'; protected $receiptUrl = 'https://www.platron.ru/receipt.php'; protected $statusUrl = 'https://www.platron.ru/get_status.php'; public function __construct(string $merchant, string $key, ?string $payUrl = null, ?string $captureUrl = null, ?string $reverseUrl = null, ?string $receiptUrl = null, ?string $statusUrl = null) { $this->merchant = $merchant; $this->key = $key; $this->payUrl = $payUrl ?? $this->payUrl; $this->captureUrl = $captureUrl ?? $this->captureUrl; $this->reverseUrl = $reverseUrl ?? $this->reverseUrl; $this->receiptUrl = $receiptUrl ?? $this->receiptUrl; $this->statusUrl = $statusUrl ?? $this->statusUrl; } public function getFormData(float $amount, string $currency, string $order, string $description, string $successUrl, string $failUrl) { $amount = sprintf('%.2f', $amount); $data = [ 'pg_amount' => $amount, 'pg_description' => $description, 'pg_order_id' => $order, 'pg_currency' => $currency, 'pg_success_url' => $successUrl, 'pg_failure_url' => $failUrl, 'pg_lifetime' => static::PAY_TTL, ]; if (defined('APPMODE') && 'dev' === APPMODE) { $data['pg_testing_mode'] = 1; } $response = $this->request($this->payUrl, $data); $return = [ 'redirect' => (string)$response->pg_redirect_url, 'transaction' => (string)$response->pg_payment_id, ]; return $return; } private function makeSigStr(string $strScriptName, array $arrParams) { unset($arrParams['pg_sig']); ksort($arrParams); return $strScriptName . ';' . $this->arJoin($arrParams) . ';' . $this->key; } private function arJoin ($in) { return substr_replace($this->arJoinProcess($in, ''), '', -1); } private function arJoinProcess ($in, $str) { if (is_array($in)) { ksort($in); $s = ''; foreach ($in as $v) { $s .= $this->arJoinProcess($v, $str); } return $s; } else { return $str . $in . ';'; } } public function sign(array $data, string $url) { $script = trim(basename(parse_url($url, PHP_URL_PATH))); $str = $this->makeSigStr($script, $data); $return = md5($str); return $return; } public function formatMessage(string $root, string $url, array $data) { $data['pg_salt'] = Samo_Utils::randomString(32); $data['pg_sig'] = $this->sign($data, $url); foreach ($data as $key => $value) { if (in_array($key, ['status', 'description'])) { $data['pg_' . $key] = $value; unset($data[$key]); } } $aXml = new Samo_XmlWriter($root, $data); return (string)$aXml; } protected function request($url, $params) { $params['pg_merchant_id'] = $this->merchant; $registry = Samo_Registry::instance(); $oldLogger = null; if (isset($registry['logger'])) { $oldLogger = $registry['logger']; } $logger = Samo_Loader::load_object('WSPAY_Logger', $oldLogger); $logger->init(); $logger->setMethod(__METHOD__); if (isset($params['pg_order_id'])) { $logger->setInumber($params['pg_order_id']); } $registry['logger'] = $logger; $e = null; $xml = $this->formatMessage('request', $url, $params); try { $response = Samo_Curl::request( $url, $xml, 'POST', [ 'Host: ' . parse_url($url, PHP_URL_HOST), 'User-agent: SAMO', 'Accept: */*', 'Content-Type: text/xml', ] ); $logger->log($response, 'response'); if ($response) { $errorMessage = null; $internalErrors = libxml_use_internal_errors(true); $disableEntities = libxml_disable_entity_loader(true); libxml_clear_errors(); try { $res = new SimpleXMLElement(empty($response) ? '<root />' : $response, LIBXML_NONET); } catch (Exception $e) { $errorMessage = $e->getMessage(); } libxml_clear_errors(); libxml_use_internal_errors($internalErrors); libxml_disable_entity_loader($disableEntities); if ($errorMessage !== null) { throw new Samo_Exception('Unable to parse response: ' . $errorMessage); } if ((string)$res->pg_status != 'ok') { $message = 'Bad response'; if (isset($res->pg_error_description)) { $message .= '(' . (string)$res->pg_error_description . ')'; } throw new Samo_Exception($message); } } else { throw new Samo_Exception('No response from payment gateway', 400); } } catch (Exception $e) { $logger->exception($e); } $logger->commit(); if ($oldLogger) { $registry['logger'] = $oldLogger; } else { unset($registry['logger']); } if ($e) { throw $e; } return $res; } public function getStatus(string $payment_id, string $inumber) : ?array { $return = null; if ($payment_id) { $params = [ 'pg_payment_id' => $payment_id, ]; } else { $params = [ 'pg_order_id' => $inumber, ]; } $response = $this->request($this->statusUrl, $params, $inumber); if ($response) { $status = (string)$response->pg_transaction_status; $return = [ 'status' => 'wait', ]; $statusMap = [ 'ok' => 'confirm', 'revoked' => 'cancel', 'failed' => 'cancel', ]; if (isset($statusMap[$status])) { $return['status'] = $statusMap[$status]; $return['inumber'] = $inumber; } } return $return; } public function sendCheck(array $data, string $inumber) { $return = false; $params = [ 'pg_operation_type' => 'payment', 'pg_order_id' => $inumber, 'pg_items' => $data, ]; $response = $this->request($this->receiptUrl, $params, $inumber); if ($response->pg_receipt_id) { $return = true; } return $return; } public function getReceipt(array $orders) { $lines = []; foreach ($orders as $order) { $lines = [ 'pg_label' => $order['label'], 'pg_price' => $order['price'], 'pg_type' => 'service', 'pg_quantity' => 1, ]; break; } return $lines; } public function capture(string $payment_id) : bool { $params = [ 'pg_payment_id' => $payment_id, ]; $response = $this->request($this->captureUrl, $params); return isset($response->pg_clearing_refund_id); } public function reverse(string $payment_id) : bool { $params = [ 'pg_payment_id' => $payment_id, ]; $response = $this->request($this->reverseUrl, $params); return $response->pg_status == 'ok'; } public function parseCallback($data) { $action = $data['action'] ?? null; if (!$action) { $knownActions = ['check', 'capture', 'refund', 'capture_request', 'refund_request', 'result']; foreach ($knownActions as $action) { if (array_key_exists($action, $data)) { break; } } unset($data[$action]); } if (!$data || !isset($data['pg_salt']) || !isset($data['pg_sig']) || !isset($data['pg_order_id']) || (!isset($data['pg_result']) && ($action == 'result')) || (!isset($data['pg_amount']) && ($action == 'check' || $action == 'result')) ) { throw new Samo_Exception('No data!'); } $signed = $data; unset($signed['inumber']); $expected = $this->sign($signed, Samo_Request::uri()); if ($data['pg_sig'] !== $expected) { throw new Samo_Exception('Invalid signature!'); } return [ 'action' => $action, 'data' => $signed, ]; } } 