<?php
 class Tinkoff { private $serviceUrl = 'https://securepay.tinkoff.ru/v2/'; private $terminalKey; private $secretKey; const PAY_TTL = 86400; private $logger; public function __construct(string $terminalKey, string $secretKey, ?string $serviceUrl = null) { $this->terminalKey = $terminalKey; $this->secretKey = $secretKey; $this->serviceUrl = $serviceUrl ?? $this->serviceUrl; } public function setLogger(WSPAY_Logger $logger = null) { if ($logger != null) { $this->logger = $logger; } return $this->logger; } public function getFormData(float $amount, string $inumber, string $description, string $successUrl, string $failUrl, bool $hold, string $email, ?array $receiptData) : ?array { $amount = ceil($amount * 100); $params = [ 'OrderId' => $inumber, 'Amount' => $amount, 'SuccessURL' => $successUrl, 'FailURL' => $failUrl, 'PayType' => $hold ? 'T' : 'O', 'Description' => $description, 'DATA' => [ 'Email' => $email, ], ]; if ($receiptData) { $params['Receipt'] = $receiptData; } $response = $this->doRequest('Init', $params, $inumber); if (is_array($response) && isset($response['Success'], $response['Status'], $response['PaymentId'], $response['PaymentURL']) && $response['Success'] && $response['Status'] === 'NEW') { return [ 'transaction' => $response['PaymentId'], 'redirect' => $response['PaymentURL'], ]; } return false; } public function resend($args) { return $this->doRequest('Resend', $args); } public function getStatus(string $payment_id, string $inumber) : ?array { $return = null; $params = [ 'PaymentId' => $payment_id, ]; $response = $this->doRequest('GetState', $params, $inumber); if ($response) { if ($response['OrderId'] != $inumber) { throw new Samo_Exception('Invalid INumber'); } $return = [ 'status' => 'wait', ]; $statusMap = [ 'AUTHORIZED' => 'hold', 'CONFIRMED' => 'confirm', 'REJECTED' => 'cancel', ]; if (isset($response['Status'], $statusMap[$response['Status']])) { $return['status'] = $statusMap[$response['Status']]; $return['inumber'] = $response['OrderId']; } } return $return; } public function capture(string $payment_id, string $inumber) : bool { $params = [ 'PaymentId' => $payment_id, ]; $response = $this->doRequest('Confirm', $params, $inumber); return is_array($response) && isset($response['Success'], $response['Status']) && $response['Success'] && $response['Status'] == 'CONFIRMED'; } public function reverse(string $payment_id, string $inumber) : bool { $params = [ 'PaymentId' => $payment_id, ]; $response = $this->doRequest('Cancel', $params, $inumber); return is_array($response) && isset($response['Success'], $response['Status']) && $response['Success'] && $response['Status'] == 'REVERSED'; } private function doRequest(string $action, array $params, string $inumber) { $oldLogger = null; $registry = Samo_Registry::instance(); if (isset($registry['logger'])) { $oldLogger = $registry['logger']; } if ($this->logger === null) { $logger = Samo_Loader::load_object('WSPAY_Logger', $oldLogger); $logger->init(); $logger->setMethod($action); $logger->setInumber($inumber); } else { $logger = $this->logger; $logger->log(['action' => $action], __METHOD__); } $registry['logger'] = $logger; $e = null; $return = null; try { $url = $this->serviceUrl; if (is_array($params)) { if (!array_key_exists('TerminalKey', $params)) { $params['TerminalKey'] = $this->terminalKey; } if (!array_key_exists('Token', $params)) { $params['Token'] = $this->genToken($params); } } $url = $this->combineUrl($url, $action); $logger->log($url, $action . '_Request'); if (is_array($params)) { $params = json_encode($params); } $headers = [ 'Content-Type: application/json', 'User-Agent: SAMO', ]; $return = null; try { $response = Samo_Curl::request($url, $params, 'POST', $headers); $return = json_decode($response, true); } catch (Throwable $e) { throw new Samo_Exception('Can not create connection to ' . $url . ' with args ' . $params, 404, $e); } $logger->log($return, $action . '_Response'); } catch (Exception $e) { $logger->exception($e); } $logger->commit(); if ($oldLogger) { $registry['logger'] = $oldLogger; } else { unset($registry['logger']); } if ($e) { throw $e; } return $return; } public function compareTokens($data) { $return = false; $notifyToken = $data['Token'] ?: null; if ($notifyToken) { unset($data['inumber'], $data['Token']); $generatedToken = $this->genToken($data); $return = $notifyToken === $generatedToken; } return $return; } public function genToken($args) { $token = ''; $args['Password'] = $this->secretKey; ksort($args); foreach ($args as $arg) { if (is_scalar($arg)) { $value = is_bool($arg) ? (true === $arg ? 'true' : 'false') : (string)$arg; $token .= $value; } } $token = hash('sha256', $token); return $token; } private function combineUrl(...$args) { $url = ''; foreach ($args as $arg) { if (is_string($arg)) { if (substr($arg, -1, 1) !== '/') { $arg .= '/'; } $url .= $arg; } else { continue; } } return $url; } public function parseCallback($raw) { $data = json_decode($raw, true); $json_error = json_last_error(); if ($json_error) { throw new Samo_Exception(json_last_error_msg(), Samo_Response::HTTP_BAD_REQUEST); } if (!isset($data['Token']) || !$this->compareTokens($data)) { throw new Samo_Exception('Wrong Token'); } return $data; } public function getReceipt(array $orders, string $vat, string $tax, string $paymentMethod, string $paymentSubject, string $email, string $phone) { $lines = []; foreach ($orders as $order) { $orderPrice = ceil($order['price'] * 100); $lines[] = [ 'Name' => $order['label'], 'Price' => $orderPrice, 'Quantity' => 1, 'Amount' => $orderPrice, 'PaymentMethod' => $paymentMethod, 'PaymentObject' => $paymentSubject, 'Tax' => $vat, ]; } $receipt = [ 'Phone' => $phone, 'Email' => $email, 'Taxation' => $tax, 'Items' => $lines, ]; return $receipt; } } 