<?php
 class Sberbank_Model extends Samo\BaseAcquiring { private const ACTION_CHECK = 'check'; private const ACTION_PAYMENT = 'payment'; private const CURRENCY = 643; private const CURRENCY_ALIAS = 'RUB'; protected const TAX_TYPE_NO_NDS = 0; protected const TAX_TYPE_NDS0 = 1; protected const TAX_TYPE_NDS10 = 2; protected const TAX_TYPE_NDS18 = 3; protected const TAX_TYPE_NDS20 = 6; protected const PAYMENT_METHOD = 1; protected const PAYMENT_OBJECT = 4; protected const STR_OK = 'OK'; protected const STR_UNKNOWN = 'UNKNOWN'; protected const CODE_ERROR = -1; protected const CODE_OK = 0; protected const CODE_UNKNOWN_ACTION = 2; protected const CODE_CLAIM_NOT_FOUND = 3; protected const CODE_BAD_SUM = 4; protected const CODE_BAD_TRANSACTION_ID = 5; protected const CODE_BAD_PAY_DATE = 6; protected const CODE_TRANSACTION_ALREADY_EXISTS = 8; protected const CODE_NOT_CONFIRMED = 12; protected static $_messages = []; protected $moduleName = 'sberbank'; public const PAY_VARIANT_ALIAS = 'sberbank'; public static function getWorkhorse(array $bankConfig) : Sberbank { return Samo_Loader::load_object( Sberbank::class, $bankConfig['SBERBANK_USERNAME'], $bankConfig['SBERBANK_PASSWORD'], $bankConfig['SBERBANK_SERVICE_URL'] ?? $bankConfig['SBERBANK_URL'] ); } protected static function orderIdFromTransaction($tr) { if (strlen($tr) == 32) { $tr = substr($tr, 0, 8) . '-' . substr($tr, 8, 4) . '-' . substr($tr, 12, 4) . '-' . substr($tr, 16, 4) . '-' . substr($tr, 20); } return $tr; } protected static function getTaxType($data) { $return = static::TAX_TYPE_NO_NDS; $orderInfo = Samo_Array::first($data); if (isset($orderInfo['StateInc']) && isset($orderInfo['StateFromInc'])) { if ($orderInfo['StateInc'] == $orderInfo['StateFromInc']) { $return = static::TAX_TYPE_NDS20; } } return $return; } public function makeFormData() : ?string { $bankConfig = static::bankConfig($this->_claim); $return = null; $inumber = $this->getInvoice()['inumber']; $logger = $this->initLogger(__METHOD__, $inumber, $this->_claim); $e = null; try { $subject = sprintf($this->messages['ACQUIRING_PAYMENT_PURPOSE'], $this->_claim); $preAuth = $bankConfig['SBERBANK_PRE_AUTH']; $receiptData = $this->getReceipt($inumber); $logger->log($receiptData, 'receipt'); $sberbank = $this->getWorkhorse($bankConfig); $res = $sberbank->getFormData($this->_amount, $bankConfig['SBERBANK_CURRENCY'] ?? null, $inumber, $subject, $this->getReturnURL($inumber), $preAuth, $receiptData); if ($res) { $transaction = $res['transaction']; self::setInvoiceExternalData($inumber, ['orderId' => $transaction]); $params = [ $inumber, $transaction, ]; $class = get_class($this); $this->createCheckOrderStatusJob( '
                Samo_Loader::load_class("' . $class . '");
                $result = call_user_func_array(["' . $class . '", "processOrderStatus"], ' . var_export($params, true) . ');
                return $result;
                ', $sberbank::PAY_TTL ); $return = $res['redirect']; } } catch (Exception $e) { $logger->exception($e); } $logger->log($return, 'result'); $logger->commit(); if ($e) { throw $e; } return $return; } public static function processOrderStatus(string $inumber, string $payment_id, ?string &$status = null) : bool { $return = false; $bankConfig = static::bankConfig(null, $inumber); $sberbank = self::getWorkhorse($bankConfig); $res = $sberbank->getStatus($payment_id, $inumber); if ($res) { $bank = $bankConfig['Inc']; $status = $res['status']; switch ($status) { case 'hold': $return = static::setHoldInvoiceByINumber($bank, $inumber); break; case 'confirm': $return = static::confirmOrder($bank, $inumber); break; case 'wait': throw new Samo_Exception(Samo_EPlatez::messages('ACQUIRING_PAYMENT_NOT_CONFIRMED')); break; case 'cancel': $return = true; break; } } return $return; } protected function getReceipt(string $inumber) { $data = static::getPaymentInfoData($inumber); $taxType = static::getTaxType($data); $email = $phone = ''; $customerDetails = []; $items = []; foreach ($data as $idx => $d) { if (!$email && $d['payee_email'] && Samo_Validate::email($d['payee_email'])) { $email = $d['payee_email']; } if (!$phone && $d['payee_phones']) { $phone = $d['payee_phones']; } $price100 = $d['price_precision'] ?? (int)($d['price'] * 100); $position = $idx + 1; $items[] = [ 'positionId' => $position, 'name' => $d['label'], 'quantity' => [ 'value' => 1.0, 'measure' => Samo_EPlatez::messages('ACQUIRING_MEASURE'), ], 'itemAmount' => $price100, 'itemCode' => $position, 'itemAttributes' => [ 'attributes' => [ [ 'name' => 'paymentMethod', 'value' => static::PAYMENT_METHOD, ], [ 'name' => 'paymentObject', 'value' => static::PAYMENT_OBJECT, ], ], ], 'tax' => [ 'taxType' => $taxType, ], 'itemPrice' => $price100, ]; } $receipt = null; if ($email || $phone) { if ($email) { $customerDetails['email'] = $email; } if ($phone) { $customerDetails['phone'] = $phone; } $receipt = [ 'customerDetails' => $customerDetails, 'cartItems' => [ 'items' => $items, ], ]; } return $receipt; } public function parseCallback() { $payment_id = Samo_Request::strval('orderId'); $inumber = Samo_Request::strval('inumber'); if (!$payment_id || !$inumber) { throw new Samo_Exception(Samo_EPlatez::messages('SBERBANK_RESULT_ERROR')); } $status = null; try { static::processOrderStatus($inumber, $payment_id, $status); $messageMap = [ 'cancel' => 'FAIL', 'hold' => 'WAIT', 'confirm' => 'OK', 'wait' => 'WAIT', ]; $resultMessage = $messageMap[$status] ?? 'FAIL'; } catch (Samo_Exception $e) { $resultMessage = 'WAIT'; } $return = [ 'inumber' => $inumber, 'result' => $resultMessage, ]; return $return; } public static function parseCallbackNotify($data, WSPAY_Logger $logger) : string { $response = Samo_Registry::get('response'); $response->respond_to('xml'); $logger->setMethod(__METHOD__); $claim = null; $inumber = null; $sberbank = Samo_Loader::load_object( Sberbank::class, '', '', '' ); if (!isset($data['ACCOUNT'])) { $return = ['CODE' => self::CODE_CLAIM_NOT_FOUND, 'MESSAGE' => 'Unexpected request']; } else { if ($ACCOUNT = $data['ACCOUNT']) { if (is_numeric($ACCOUNT)) { $claim = (int)$ACCOUNT; } else { $inumber = $ACCOUNT; } } $return = []; try { if (isset($data['salt'], $data['psw'], $data['login'])) { $bankConfig = static::bankConfig($claim, $inumber,null, $data['login']); if ($data['login'] !== $bankConfig['Login'] || md5($data['salt'] . $bankConfig['Psw']) !== $data['psw']) { throw new Samo_Exception(sprintf(Samo_EPlatez::messages('ACQUIRING_BANK_LOGIN_NOT_FOUND'), $data['login']), self::CODE_NOT_CONFIRMED); } } else { if (isset($_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW'])) { $user = trim($_SERVER['PHP_AUTH_USER']); $pass = trim($_SERVER['PHP_AUTH_PW']); if (!$user || !$pass) { $logger->log('Wrong Username or Password!', 'return'); $logger->commit(); return $response->unauthorized('Wrong Username or Password!', 'SBERBANK Login'); } $bankConfig = static::bankConfig($claim, $inumber,null, $user); if ($bankConfig['Login'] != $user || $bankConfig['Psw'] != $pass) { throw new Samo_Exception(sprintf(Samo_EPlatez::messages('ACQUIRING_BANK_LOGIN_NOT_FOUND'), $user), self::CODE_NOT_CONFIRMED); } } else { $logger->log('Enter Username and Password!', 'return'); $logger->commit(); return $response->unauthorized('Enter Username and Password!', 'SBERBANK Login'); } } $sberbank = static::getWorkhorse($bankConfig); $ACTION = $data['ACTION']; if (!in_array($ACTION, [self::ACTION_CHECK, self::ACTION_PAYMENT])) { throw new Samo_Exception(Samo_EPlatez::messages('ACQUIRING_UNKNOWN_REQUEST_TYPE'), self::CODE_UNKNOWN_ACTION); } $model = static::_getModel($bankConfig['Inc']); if ($ACTION == 'check') { $return = static::checkSberOnline($model, $claim, $inumber); } if ($ACTION == 'payment') { $amount = (float)$data['AMOUNT']; $operationType = $data['PAY_CHAN'] ?? null; if ($claim) { $Transaction = $data['PAY_ID']; $result = $model->payClaim($claim, null, $amount, $Transaction, null, null, null, $operationType); $dt = $result['ConfirmDateTime']; $id = $result['INumber']; } else { $invoice = $model->getInvoiceData($inumber); if (floor($amount) != floor($invoice['RubSum'])) { throw new Samo_Exception(Samo_EPlatez::messages('ACQUIRING_INCORRECT_AMOUNT'), self::CODE_BAD_SUM); } $model->confirmInvoiceByINumber($ACCOUNT, PSBANK_DEFAULT_OPERATION, $operationType); $dt = Samo_Datetime::now(); $id = $inumber; } $return = [ 'CODE' => self::CODE_OK, 'MESSAGE' => self::STR_OK, 'REG_DATE' => $dt->format('d.m.Y_H:i:s'), 'PAY_ID_EXT' => $id, ]; } } catch (WSPAY_exception $e) { $logger->exception($e); switch ($e->getCode()) { case WSPAY_model::BANK_NOT_FOUND: $code = self::CODE_NOT_CONFIRMED; break; case WSPAY_model::CLAIM_NOT_FOUND: case WSPAY_model::CLAIM_OR_PASSPORT_INCORRECT: case WSPAY_model::TICKET_NOT_FOUND: case WSPAY_model::TICKET_ACCESS_DENIED: $code = self::CODE_CLAIM_NOT_FOUND; break; case WSPAY_model::PAYMENT_MINUS: $code = ACTION == self::ACTION_PAYMENT ? self::CODE_BAD_SUM : self::CODE_ERROR; break; case WSPAY_model::TICKET_CONFIRMED: case WSPAY_model::DOUBLE_TICKET: case WSPAY_model::CLAIM_FULLY_PAID: $code = $ACTION == self::ACTION_PAYMENT ? self::CODE_TRANSACTION_ALREADY_EXISTS : self::CODE_ERROR; break; default: $code = self::CODE_ERROR; break; } $return = ['CODE' => $code, 'MESSAGE' => $e->getMessage()]; } catch (Samo_Exception $e) { $logger->exception($e); $code = $e->getCode(); if (!in_array($code, [self::CODE_UNKNOWN_ACTION, self::CODE_CLAIM_NOT_FOUND, self::CODE_BAD_SUM, self::CODE_BAD_TRANSACTION_ID, self::CODE_BAD_PAY_DATE, self::CODE_TRANSACTION_ALREADY_EXISTS, self::CODE_NOT_CONFIRMED])) { $code = self::CODE_ERROR; } $return = ['CODE' => $code, 'MESSAGE' => $e->getMessage()]; } } $response = $sberbank->formatMessage($return); return $response; } static protected function checkSberOnline(WSPAY_model $model, ?int $claim = null, ?string $inumber = null) { $payMinAmount = 0; $ownerrs = ''; $ownerinn = ''; $ownername = ''; if ($claim) { $result = $model->getClaim($claim); $fio = $result['BuyerName'] ?: $result['PeopleLName']; $balance = $result['PayDebt']; $payMinAmount = $result['PayMinAmount']; $ownerrs = $result['OwnerRS']; $ownerinn = $result['OwnerINN']; $ownername = $result['OwnerOfficialName']; } else { $invoice = $model->getInvoiceData($inumber); $fio = $invoice['Payer']; $balance = $invoice['RubSum']; } $return = [ 'CODE' => self::CODE_OK, 'MESSAGE' => self::STR_OK, 'FIO' => trim($fio) ? $fio : 'UNKNOWN', 'ADDRESS' => 'UNKNOWN', 'ACCOUNT_BALANCE' => str_replace(',', '.', (string)($balance)), 'MIN_AMOUNT' => str_replace(',', '.', (string)($payMinAmount)), 'RASCH' => $ownerrs, 'INN' => $ownerinn, 'NAZN' => $ownername, ]; return $return; } protected function _reverse($invoice) { $bankConfig = static::bankConfig(null, $invoice['INumber']); $payment_id = $invoice['ExternalData']['orderId'] ?? self::orderIdFromTransaction($invoice['TransactionId']); if ($payment_id) { $sberbank = $this->getWorkhorse($bankConfig); $sberbank->reverse($payment_id, $invoice['INumber']); return true; } return false; } protected function _capture($invoice) { $bankConfig = static::bankConfig(null, $invoice['INumber']); $payment_id = $invoice['ExternalData']['orderId'] ?? self::orderIdFromTransaction($invoice['TransactionId']); if ($payment_id) { $sberbank = $this->getWorkhorse($bankConfig); $sberbank->capture($payment_id, $invoice['INumber']); return true; } return false; } } 